<?php

use QUI\System\VhostManager;
use QUI\ERP\Payments\Stripe\Provider;
use Stripe\ApplePayDomain;
use Stripe\Stripe as StripeApiClient;

/**
 * Register a domain with Apple Pay
 *
 * @param string $domain
 * @param bool $unregister (optional) - UNregister domain [default: false]
 * @return bool - success
 */
QUI::$Ajax->registerFunction(
    'package_quiqqer_payment-stripe_ajax_backend_settings_registerApplePayDomain',
    function ($domain, $unregister = null) {
        $VhostManager = new VhostManager();
        $vhosts       = $VhostManager->getList();

        if (!isset($vhosts[$domain])) {
            return false;
        }

        $domain       = \str_replace(['http://', 'https://'], '', $domain);
        $clientSecret = Provider::getClientSecret(false);
        $unregister   = boolval($unregister);

        if (empty($clientSecret)) {
            QUI::getMessagesHandler()->addError(
                QUI::getLocale()->get(
                    'quiqqer/payment-stripe',
                    'message.ajax.backend.settings.registerApplePayDomain.error.secret_key_required'
                )
            );

            return false;
        }

        $register = function () use ($clientSecret, $domain, $unregister) {
            // Setup API with LIVE secret
            StripeApiClient::setApiKey($clientSecret);

            if ($unregister) {
                $applePayDomains  = ApplePayDomain::all()->toArray();
                $applePayDomains  = $applePayDomains['data'];
                $applePayDomainId = false;

                foreach ($applePayDomains as $data) {
                    if ($data['domain_name'] === $domain) {
                        $applePayDomainId = $data['id'];
                        break;
                    }
                }

                if ($applePayDomainId) {
                    $ApplePayDomain = ApplePayDomain::retrieve($applePayDomainId);
                    $ApplePayDomain->delete();
                }
            } else {
                ApplePayDomain::create([
                    'domain_name' => $domain
                ]);
            }

            // Save in conf
            $Conf       = QUI::getPackage('quiqqer/payment-stripe')->getConfig();
            $registered = $Conf->get('payment', 'apple_pay_domains');

            if (empty($registered)) {
                $registered = [];
            } else {
                $registered = \json_decode($registered, true);
            }

            $registered[$domain] = $unregister ? 0 : 1;
            $Conf->set('payment', 'apple_pay_domains', \json_encode($registered));
            $Conf->save();
        };

        try {
            $register();
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            if (\mb_strpos($Exception->getMessage(), 'You have an outdated file hosted') !== false) {
                Provider::updateApplePayDomainAssociationFile();
                $register();
            } else {
                QUI::getMessagesHandler()->addError(
                    QUI::getLocale()->get(
                        'quiqqer/payment-stripe',
                        'message.ajax.backend.settings.registerApplePayDomain.error',
                        [
                            'domain' => $domain,
                            'error'  => $Exception->getMessage()
                        ]
                    )
                );
            }

            return false;
        }

        if ($unregister) {
            QUI::getMessagesHandler()->addSuccess(
                QUI::getLocale()->get(
                    'quiqqer/payment-stripe',
                    'message.ajax.backend.settings.registerApplePayDomain.success_unregister',
                    [
                        'domain' => $domain
                    ]
                )
            );
        } else {
            QUI::getMessagesHandler()->addSuccess(
                QUI::getLocale()->get(
                    'quiqqer/payment-stripe',
                    'message.ajax.backend.settings.registerApplePayDomain.success',
                    [
                        'domain' => $domain
                    ]
                )
            );
        }

        return true;
    },
    ['domain', 'unregister'],
    ['Permission::checkAdminUser']
);
