<?php

use QUI\ERP\Order\Handler;
use QUI\Utils\Security\Orthos;
use QUI\ERP\Payments\Stripe\StripeException;
use QUI\ERP\Payments\Stripe\AbstractBasePayment;
use QUI\ERP\Payments\Stripe\PaymentMethods\Recurring\Subscriptions;
use Stripe\Subscription as StripeSubscription;
use Stripe\Invoice as StripeInvoice;
use QUI\ERP\Payments\Stripe\Utils;

/**
 * Create a Stripe PaymentIntent
 *
 * @param int $basketId - Basket ID
 * @return array - Subscription confirmation data
 */
QUI::$Ajax->registerFunction(
    'package_quiqqer_payment-stripe_ajax_createSubscription',
    function ($orderHash, $paymentMethodId, $resetPaymentMethod = null) {
        try {
            $orderHash       = Orthos::clear($orderHash);
            $paymentMethodId = Orthos::clear($paymentMethodId);
            $Order           = Handler::getInstance()->getOrderByHash($orderHash);

            /** @var \QUI\ERP\Payments\Stripe\PaymentMethods\Recurring\AbstractBaseRecurringPayment $Payment */
            $Payment = $Order->getPayment()->getPaymentType();

            // Save payment id in order
            $Order->setPaymentData(AbstractBasePayment::ATTR_STRIPE_PAYMENT_METHOD_ID, $paymentMethodId);
            $Order->update(QUI::getUsers()->getSystemUser());

            // Create Stripe Subscription (and Plan if necessary)
            $subscriptionId = $Payment->createSubscription($Order);

            /**
             * The process to re-set a payment method (e.g. if the user authentication failed) consists
             * of the following steps:
             *
             * 1. Add newly generated PaymentMethod to the Stripe Customer
             * 2. Set the PaymentMethod as the Subscription default
             * 3. Try to pay the latest Subscription Invoice -> This can fail initially if the new PaymentMethod requires
             * additional authentication.
             * 4. Confirm Subscription
             */
            if (!empty($resetPaymentMethod)) {
                Utils::attachPaymentMethodToStripeCustomer($Order->getCustomer(), $paymentMethodId);

                StripeSubscription::update(
                    $subscriptionId,
                    [
                        'default_payment_method' => $paymentMethodId
                    ]
                );

                try {
                    StripeInvoice::retrieve(Subscriptions::getLatestSubscriptionInvoiceId($subscriptionId))->pay();
                } catch (\Exception $Exception) {
                    QUI\System\Log::writeDebugException($Exception);
                }
            }

            return Subscriptions::confirmSubscription($subscriptionId);
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            throw new StripeException([
                'quiqqer/payment-stripe',
                'exception.ajax.createPaymentIntent.general_error'
            ]);
        }
    },
    ['orderHash', 'paymentMethodId', 'resetPaymentMethod'],
    'Permission::checkUser'
);
